/*** readypin.c  -*- C -*- */
/** Blink a LED so long a button is pressed. */

/*** Ivan Shmakov, 2022 */

/** To the extent possible under law, the author(s) have dedicated
 ** all copyright and related and neighboring rights to this software
 ** to the public domain worldwide.  This software is distributed
 ** without any warranty.
 **
 ** You should have received a copy of the CC0 Public Domain Dedication
 ** along with this software.  If not, see
 ** <http://creativecommons.org/publicdomain/zero/1.0/>.
 */

/*** History: */

/** 0.1 2022-10-04
 **     Initial revision.
 */

/*** Code: */
#include <stdint.h>

#include <machine.h>
#include <readyled.h>

/** This section defines the button */
#ifndef READYPIN_BUTTON_SHIFT
#error "READYPIN_BUTTON_SHIFT must be defined; use 0 for PA0, 10 for PB2, etc."
#endif
/** NB: compiler optimizations may get ruined if this is not 1 */
#ifndef READYPIN_BUTTON_MASK
#define READYPIN_BUTTON_MASK    (1ULL)
#endif
/** This can be used to activate the internal pull-ups on AVR */
#ifndef READYPIN_BUTTON_WRITE
#define READYPIN_BUTTON_WRITE   (0ULL)
#endif
/** And this can be used to make the input pin active-low */
#ifndef READYPIN_BUTTON_XOR
#define READYPIN_BUTTON_XOR     READYPIN_BUTTON_WRITE
#endif

/** NB: the time base is generally F_CPU / 256 on AVR MCUs */
#ifndef READYPIN_TIMER_DIVISOR
#define READYPIN_TIMER_DIVISOR  (1024L)
#endif

int
main ()
{
  machine_init ();
  machine_timer_setup (READYPIN_TIMER_DIVISOR);

  machine_port_setup ((READYPIN_BUTTON_MASK << READYPIN_BUTTON_SHIFT),
                      (READYLED_PORT_MASK));
#if READYPIN_BUTTON_WRITE
  /** Activate pull-ups on AVR */
  machine_port_write ((READYPIN_BUTTON_WRITE),
                      (READYPIN_BUTTON_WRITE));
#endif

  while (1) {
    machine_wait ();

    if (machine_timer_ticked_p ()) {
      readyled_tick ();
    }
    machine_port_map_t v
      = machine_port_read  (READYPIN_BUTTON_SHIFT,
                            READYPIN_BUTTON_MASK);
    if ((v ^ READYPIN_BUTTON_XOR)) {
      readyled_blink ();
    }
  }

  /* not reached */
}

/*** readypin.c ends here */
