/*** readyled.h  -*- C -*- */
/** Blinking Ready LED. */

/*** Ivan Shmakov, 2019, 2022 */

/** To the extent possible under law, the author(s) have dedicated
 ** all copyright and related and neighboring rights to this software
 ** to the public domain worldwide.  This software is distributed
 ** without any warranty.
 **
 ** You should have received a copy of the CC0 Public Domain Dedication
 ** along with this software.  If not, see
 ** <http://creativecommons.org/publicdomain/zero/1.0/>.
 */

/*** History: */

/** 0.2 2022-10-02
 **     Add the usual boilerplate.  Minor coding style tweaks
 **     all across the code.
 **     (READYLED_PORT_MASK): Use 1LL (was: 1L) as the constant;
 **     wrap READYLED_PORT_SHIFT into parentheses.
 **
 ** 0.1 2019-06-20 09:25:11Z
 **     (sfn.Zswt7fATQpH9FKaI7b-8--WBmq5siWkp-1XoiObvubg.h)
 **     Initial revision.
 */

/*** Code: */
#ifndef READYLED_H
#define READYLED_H

#include <machine.h>

/** Use PD3 for RDY LED by default */
#ifndef READYLED_PORT_SHIFT
#define READYLED_PORT_SHIFT (27)
#endif
#ifndef READYLED_PORT_MASK
#define READYLED_PORT_MASK  (1LL << (READYLED_PORT_SHIFT))
#endif
#ifndef READYLED_PORT_INVERSE
#define READYLED_PORT_INVERSE (1)
#endif
#ifndef READYLED_PORT_ON
#define READYLED_PORT_ON  ((READYLED_PORT_INVERSE) ? 0 : (READYLED_PORT_MASK))
#endif
#ifndef READYLED_PORT_OFF
#define READYLED_PORT_OFF ((READYLED_PORT_ON) ^ (READYLED_PORT_MASK))
#endif

/** Turn RDY LED off for DELAY ticks, then ensure off for - PAUSE */
#ifndef READYLED_DELAY
#define READYLED_DELAY   (3)
#endif
#ifndef READYLED_PAUSE
#define READYLED_PAUSE   (-7)
#endif

static int_least8_t readyled_wait = 0;

static void
readyled_tick (void)
{
  if (readyled_wait < READYLED_PAUSE || --readyled_wait != 0) {
    /* . */
    return;
  }
  machine_port_write (READYLED_PORT_ON, READYLED_PORT_MASK);
  /* . */
}

static void
readyled_blink (void)
{
  if (readyled_wait > READYLED_PAUSE) {
    /* . */
    return;
  }
  readyled_wait
    = READYLED_DELAY;
  machine_port_write (READYLED_PORT_OFF, READYLED_PORT_MASK);
  /* . */
}

#endif
/*** readyled.h ends here */
